@@ -1,5 +1,5 @@
 /*
-        Copyright (C) 1993-2005 Hewlett-Packard Company
+        Copyright (C) 1993-2012 Hewlett-Packard Company
 */
 
 #ifdef HAVE_CONFIG_H
@@ -18,7 +18,7 @@
 
 #define PAD_TIME 4
 /* library routine specifc defines                                      */
-#define         MAXSPECDATA     62      /* how many ints worth of data  */
+#define         MAXSPECDATA     162     /* how many ints worth of data  */
                                         /* can tests send...            */
 #define         MAXTIMES        4       /* how many times may we loop   */
                                         /* to calibrate                 */
@@ -33,6 +33,7 @@
 #define         DEBUG_OK                3
 #define         NODE_IDENTIFY           4
 #define         CPU_CALIBRATE           5
+#define         PASSPHRASE              6
 
 #define         DO_TCP_STREAM           10
 #define         TCP_STREAM_RESPONSE     11
@@ -233,6 +234,10 @@
 #else
 # if HAVE_STDINT_H
 #  include <stdint.h>
+#else
+# ifdef WIN32
+#  include "missing\inttypes.h"
+# endif
 # endif
 #endif
 
@@ -241,6 +246,12 @@ enum sock_buffer{
   RECV_BUFFER
 };
 
+enum netperf_output_modes {
+  HUMAN = 0,
+  CSV,
+  KEYVAL,
+};
+
 /* some defines for security types, perhaps these would be better
    elsewhere but for now here they are */
 
@@ -283,6 +294,7 @@ struct ring_elt {
   struct ring_elt *next;  /* next element in the ring */
   char *buffer_base;      /* in case we have to free it at somepoint */
   char *buffer_ptr;       /* the aligned and offset pointer */
+  void *completion_ptr;   /* a pointer to information for async completion */
 };
 
 /* +*+ SAF  Sorry about the hacks with errno; NT made me do it :(
@@ -479,6 +491,23 @@ extern  int     times_up;
 extern  FILE    *where;
 extern  int     loops_per_msec;
 extern  float   lib_local_per_cpu_util[];
+
+extern enum netperf_output_modes netperf_output_mode;
+
+#if defined(WANT_INTERVALS) || defined(WANT_DEMO)
+
+extern int    demo_mode;
+extern double demo_interval;
+extern double demo_units;
+extern double units_this_tick;
+#if defined(WANT_DEMO)
+extern void   demo_rr_interval(uint32_t units);
+extern void   demo_rr_setup(uint32_t units);
+extern void   demo_stream_interval(uint32_t units);
+extern void   demo_interval_tick(uint32_t units);
+extern void   demo_interval_final();
+#endif
+#endif 
   
 extern  void    netlib_init();
 extern  int     netlib_get_page_size();
@@ -498,10 +527,12 @@ extern  void    send_request();
 extern  void    recv_response();
 extern  void    send_response();
 extern  int     recv_request();
+extern  int     recv_request_timed_n(int n, int seconds);
 extern  void    send_request_n(int n);  /* convert only the first N ints */
 extern  void    recv_response_n(int n); /* of the test-specific data via */
 extern  void    send_response_n(int n); /* htonl/ntonl as required */
 extern  int     recv_request_n(int n);
+extern  void    fixup_request_n(int n);
 extern  void    dump_request();
 extern  void    dump_addrinfo(FILE *dumploc, struct addrinfo *info,
 			      char *host, char *port, int family);
@@ -560,7 +591,11 @@ extern char     *find_interface_slot(char *interface_name);
 extern void     find_interface_ids(char *interface_name, int *vendor, int *device, int *sub_vend, int *sub_dev);
 extern void     find_driver_info(char *ifname, char *driver, char *version, char *firmware, char *bus, int len);
 extern void     find_system_info(char **system_model, char **cpu_model, int *cpu_frequency);
-
+extern int      HIST_get_percentile();
+extern void     HIST_get_stats();
+extern void     find_security_info(int *enabled, int *type, char **specific);
+extern void     demo_first_timestamp();
+extern void     demo_stream_setup(uint32_t a, uint32_t b);
 #ifndef WIN32
 
 /* WIN32 requires that at least one of the file sets to select be
@@ -611,6 +646,7 @@ extern unsigned int convert_timespec(char *string);
 
 #ifdef WANT_INTERVALS
 extern void start_itimer(unsigned int interval_len_msec);
+extern void stop_itimer(void);
 #endif
  /* these are all for the confidence interval stuff */
 extern double confidence;
@@ -627,6 +663,7 @@ extern int lib_cpu_map[];
 #endif
 
 #ifdef WIN32
+extern HANDLE WinTimer;
 #if 0
 /* Should really use safe string functions; but not for now... */
 #include <strsafe.h>
@@ -635,6 +672,7 @@ extern int lib_cpu_map[];
 #endif
 
 #define snprintf _snprintf
+#define strdup   _strdup
 #endif
 
 /* Define a macro to align a buffer with an offset from a power of 2
