@@ -1,30 +1,30 @@
 /*
- 
-	   Copyright (C) 1993-2011 Hewlett-Packard Company
+
+	   Copyright (C) 1993-2012 Hewlett-Packard Company
                          ALL RIGHTS RESERVED.
- 
+
   The enclosed software and documentation includes copyrighted works
   of Hewlett-Packard Co. For as long as you comply with the following
   limitations, you are hereby authorized to (i) use, reproduce, and
   modify the software and documentation, and to (ii) distribute the
   software and documentation, including modifications, for
   non-commercial purposes only.
-      
+
   1.  The enclosed software and documentation is made available at no
       charge in order to advance the general development of
       high-performance networking products.
- 
+
   2.  You may not delete any copyright notices contained in the
       software or documentation. All hard copies, and copies in
       source code or object code form, of the software or
       documentation (including modifications) must contain at least
       one of the copyright notices.
- 
+
   3.  The enclosed software and documentation has not been subjected
       to testing and quality control and is not a Hewlett-Packard Co.
       product. At a future time, Hewlett-Packard Co. may or may not
       offer a version of the software and documentation as a product.
-  
+
   4.  THE SOFTWARE AND DOCUMENTATION IS PROVIDED "AS IS".
       HEWLETT-PACKARD COMPANY DOES NOT WARRANT THAT THE USE,
       REPRODUCTION, MODIFICATION OR DISTRIBUTION OF THE SOFTWARE OR
@@ -34,18 +34,18 @@
       EXPRESS AND IMPLIED, WITH REGARD TO THE SOFTWARE AND THE
       DOCUMENTATION. HP SPECIFICALLY DISCLAIMS ALL WARRANTIES OF
       MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-  
+
   5.  HEWLETT-PACKARD COMPANY WILL NOT IN ANY EVENT BE LIABLE FOR ANY
       DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
       (INCLUDING LOST PROFITS) RELATED TO ANY USE, REPRODUCTION,
       MODIFICATION, OR DISTRIBUTION OF THE SOFTWARE OR DOCUMENTATION.
- 
+
 */
 
 #include "netperf_version.h"
 
 char	netserver_id[]="\
-@(#)netserver.c (c) Copyright 1993-2011 Hewlett-Packard Co. Version 2.5.0";
+@(#)netserver.c (c) Copyright 1993-2012 Hewlett-Packard Co. Version 2.6.0";
 
 
 #ifdef HAVE_CONFIG_H
@@ -109,11 +109,11 @@ char	netserver_id[]="\
 #ifndef SIGCLD
 #define SIGCLD SIGCHLD
 #endif /* SIGCLD */
-      
+
 #endif
 
 #if !defined(HAVE_SETSID)
-#if HAVE_SYS_WAIT_H>
+#if HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif
 #endif
@@ -122,14 +122,17 @@ char	netserver_id[]="\
 #include <time.h>
 #include <winsock2.h>
 
-#if HAVE_WS2TCPIP.h
+#if HAVE_WS2TCPIP_H
 #include <ws2tcpip.h>
 #endif
 
 #include <windows.h>
 
+#include "missing\stdint.h"
+
 #define strdup _strdup
-#define sleep(x) SLEEP((x)*1000)
+#define sleep(x) Sleep((x)*1000)
+#define netperf_socklen_t socklen_t
 #endif /* WIN32 */
 
 /* unconditional system includes */
@@ -158,7 +161,7 @@ char	netserver_id[]="\
 
 #ifndef DEBUG_LOG_FILE_DIR
 #if defined(WIN32)
-#define DEBUG_LOG_FILE_DIR "c:\\temp\\"
+#define DEBUG_LOG_FILE_DIR ""
 #elif defined(ANDROID)
 #define DEBUG_LOG_FILE_DIR "/data/local/tmp/"
 #else
@@ -175,8 +178,6 @@ char	netserver_id[]="\
 #endif
 char     FileName[PATH_MAX];
 
-
-
 char     listen_port[10];
 
 struct listen_elt {
@@ -200,6 +201,8 @@ int      suppress_debug = 0;
 extern	char	*optarg;
 extern	int	optind, opterr;
 
+/* char  *passphrase = NULL; */
+
 static void
 init_netserver_globals() {
 
@@ -229,7 +232,7 @@ unlink_empty_debug_file() {
 
   if (stat(FileName,&buf)== 0) {
 
-    if (buf.st_size == 0) 
+    if (buf.st_size == 0)
       unlink(FileName);
   }
 #endif
@@ -239,7 +242,7 @@ unlink_empty_debug_file() {
    routine as we depend on the control socket being dup()ed out of the
    way when we go messing about with the streams. */
 void
-open_debug_file() 
+open_debug_file()
 {
 #if !defined WIN32
 #define NETPERF_NULL "/dev/null"
@@ -253,7 +256,12 @@ open_debug_file()
 
   snprintf(FileName,
 	   sizeof(FileName),
+#if defined(WIN32)
+	   "%s\\%s_%d",
+	   getenv("TEMP"),
+#else
 	   "%s_%d",
+#endif
 	   DEBUG_LOG_FILE,
 	   getpid());
   if ((where = fopen((suppress_debug) ? NETPERF_NULL : FileName,
@@ -346,19 +354,19 @@ open_debug_file()
   /* Hopefully, by closing stdout & stderr, the subsequent fopen calls
      will get mapped to the correct std handles. */
   fclose(stdout);
-  
+
   if ((where = fopen(FileName, "w")) == NULL) {
     perror("netserver: fopen of debug file as new stdout failed!");
     exit(1);
   }
-  
+
   fclose(stderr);
-  
+
   if ((where = fopen(FileName, "w")) == NULL) {
     fprintf(stdout, "fopen of debug file as new stderr failed!\n");
     exit(1);
   }
-  
+
 #endif
 
 }
@@ -393,7 +401,7 @@ set_server_sock() {
   server_sock = dup(0);
 
 #else
-  if ((server_sock = 
+  if ((server_sock =
        socket(TCPIP$C_AUXS, SOCK_STREAM, 0)) == INVALID_SOCKET) {
     fprintf(stderr,
 	    "%s: failed to grab aux server socket: %s (errno %s)\n",
@@ -455,7 +463,7 @@ create_listens(char hostname[], char port[], int af) {
 
   if (error) {
     if (debug) {
-      
+
       fprintf(stderr,
 	      "%s: could not resolve remote '%s' port '%s' af %d\n"
 	      "\tgetaddrinfo returned %s (%d)\n",
@@ -465,7 +473,7 @@ create_listens(char hostname[], char port[], int af) {
 	      af,
 	      gai_strerror(error),
 	      error);
-      
+
     }
     return;
   }
@@ -495,9 +503,9 @@ create_listens(char hostname[], char port[], int af) {
 
     /* happiness and joy, keep going */
     if (setsockopt(temp_socket,
-		   SOL_SOCKET, 
-		   SO_REUSEADDR, 
-		   (char *)&on , 
+		   SOL_SOCKET,
+		   SO_REUSEADDR,
+		   (char *)&on ,
 		   sizeof(on)) == SOCKET_ERROR) {
       if (debug) {
 	fprintf(stderr,
@@ -511,7 +519,7 @@ create_listens(char hostname[], char port[], int af) {
     /* still happy and joyful */
 
     if ((bind(temp_socket,
-	      local_res_temp->ai_addr, 
+	      local_res_temp->ai_addr,
 	      local_res_temp->ai_addrlen) != SOCKET_ERROR) &&
 	(listen(temp_socket,128) != SOCKET_ERROR))  {
 
@@ -625,12 +633,13 @@ setup_listens(char name[], char port[], int af) {
   }
 }
 
-int
+SOCKET
 set_fdset(struct listen_elt *list, fd_set *fdset) {
 
-  FD_ZERO(fdset);
   struct listen_elt *temp;
-  int max = -1;
+  SOCKET max = INVALID_SOCKET;
+
+  FD_ZERO(fdset);
 
   temp = list;
 
@@ -675,24 +684,52 @@ close_listens(struct listen_elt *list) {
   }
 
   temp = list;
-  
+
   while (temp) {
     close(temp->fd);
     temp = temp->next;
   }
 }
 
+static int
+recv_passphrase() {
+
+  /* may need to revisit the timeout. we only respond if there is an
+     error with receiving the passphrase */
+  if ((recv_request_timed_n(0,20) > 0) &&
+      (netperf_request.content.request_type == PASSPHRASE) &&
+      (!strcmp(passphrase,
+	       (char *)netperf_request.content.test_specific_data))) {
+    /* it was okey dokey */
+    return 0;
+  }
+#if defined(SEND_PASSPHRASE_RESPONSE)
+  netperf_response.content.response_type = PASSPHRASE;
+  netperf_response.content.serv_errno = 403;
+    snprintf((char *)netperf_response.content.test_specific_data,
+	     sizeof(netperf_response.content.test_specific_data),
+	     "Sorry, unable to match with required passphrase\n");
+  send_response_n(0);
+#endif
+  fprintf(where,
+	  "Unable to match required passphrase.  Closing control connection\n");
+  fflush(where);
+
+  close(server_sock);
+  return -1;
+}
+
 /* This routine implements the "main event loop" of the netperf server
    code. Code above it will have set-up the control connection so it
    can just merrily go about its business, which is to "schedule"
    performance tests on the server.  */
 
-void 
+void
 process_requests()
 {
-  
+
   float	temp_rate;
-  
+
   if (debug) {
     fprintf(where,
 	    "%s: enter\n",
@@ -700,13 +737,23 @@ process_requests()
     fflush(where);
   }
 
-  while (1) {
+  /* if the netserver was started with a passphrase, look for it in
+     the first request to arrive.  if there is no passphrase in the
+     first request we will end-up dumping the control connection. raj
+     2012-01-23 */
 
-    if (recv_request() == 0) 
+  if ((passphrase != NULL)  && (recv_passphrase()))
       return;
 
+  while (1) {
+
+    if (recv_request() <= 0) {
+      close(server_sock);
+      return;
+    }
+
     switch (netperf_request.content.request_type) {
-      
+
     case DEBUG_ON:
       netperf_response.content.response_type = DEBUG_OK;
       if (!suppress_debug) {
@@ -723,7 +770,7 @@ process_requests()
 
       send_response();
       break;
-      
+
     case DEBUG_OFF:
       if (debug)
 	debug--;
@@ -743,7 +790,7 @@ process_requests()
 	delims[1] = strdup(",");
 	delims[2] = strdup("_");
 	delims[3] = strdup(";");
-	
+
 	netperf_response.content.response_type = SYSINFO_RESPONSE;
 	for (i = 0; i < 4; i++) {
 	  if ((!strstr(local_sysname,delims[i])) &&
@@ -790,7 +837,7 @@ process_requests()
 	    (char *)netperf_response.content.test_specific_data,
 	    sizeof(temp_rate));
       bcopy((char *)&lib_num_loc_cpus,
-	    (char *)netperf_response.content.test_specific_data + 
+	    (char *)netperf_response.content.test_specific_data +
 	            sizeof(temp_rate),
 	    sizeof(lib_num_loc_cpus));
       if (debug) {
@@ -800,64 +847,64 @@ process_requests()
 		lib_num_loc_cpus);
 	fflush(where);
       }
-      
+
       /* we need the cpu_start, cpu_stop in the looper case to kill
          the child proceses raj 7/95 */
-      
+
 #ifdef USE_LOOPER
       cpu_start(1);
       cpu_stop(1,&temp_rate);
 #endif /* USE_LOOPER */
-      
+
       send_response();
       break;
-      
+
     case DO_TCP_STREAM:
       recv_tcp_stream();
       break;
-      
+
     case DO_TCP_MAERTS:
       recv_tcp_maerts();
       break;
-      
+
     case DO_TCP_RR:
       recv_tcp_rr();
       break;
-      
+
     case DO_TCP_CRR:
       recv_tcp_conn_rr();
       break;
-      
+
     case DO_TCP_CC:
       recv_tcp_cc();
       break;
-      
+
 #ifdef DO_1644
     case DO_TCP_TRR:
       recv_tcp_tran_rr();
       break;
 #endif /* DO_1644 */
-      
+
 #ifdef DO_NBRR
     case DO_TCP_NBRR:
       recv_tcp_nbrr();
       break;
 #endif /* DO_NBRR */
-      
+
     case DO_UDP_STREAM:
       recv_udp_stream();
       break;
-      
+
     case DO_UDP_RR:
       recv_udp_rr();
       break;
-      
+
 #ifdef WANT_DLPI
 
     case DO_DLPI_CO_RR:
       recv_dlpi_co_rr();
       break;
-      
+
     case DO_DLPI_CL_RR:
       recv_dlpi_cl_rr();
       break;
@@ -877,34 +924,34 @@ process_requests()
     case DO_STREAM_STREAM:
       recv_stream_stream();
       break;
-      
+
     case DO_STREAM_RR:
       recv_stream_rr();
       break;
-      
+
     case DO_DG_STREAM:
       recv_dg_stream();
       break;
-      
+
     case DO_DG_RR:
       recv_dg_rr();
       break;
-      
+
 #endif /* WANT_UNIX */
 
 #ifdef WANT_XTI
     case DO_XTI_TCP_STREAM:
       recv_xti_tcp_stream();
       break;
-      
+
     case DO_XTI_TCP_RR:
       recv_xti_tcp_rr();
       break;
-      
+
     case DO_XTI_UDP_STREAM:
       recv_xti_udp_stream();
       break;
-      
+
     case DO_XTI_UDP_RR:
       recv_xti_udp_rr();
       break;
@@ -915,7 +962,7 @@ process_requests()
     case DO_SCTP_STREAM:
       recv_sctp_stream();
       break;
-      
+
     case DO_SCTP_STREAM_MANY:
       recv_sctp_stream_1toMany();
       break;
@@ -941,7 +988,7 @@ process_requests()
     case DO_SDP_RR:
       recv_sdp_rr();
       break;
-#endif 
+#endif
 
 #ifdef WANT_OMNI
     case DO_OMNI:
@@ -949,6 +996,13 @@ process_requests()
       break;
 #endif
 
+    case PASSPHRASE:
+      if (debug) {
+	fprintf(where,"Ignoring an unexpected passphrase control message\n");
+	fflush(where);
+      }
+      break;
+
     default:
       fprintf(where,"unknown test number %d\n",
 	      netperf_request.content.request_type);
@@ -956,7 +1010,7 @@ process_requests()
       netperf_response.content.serv_errno=998;
       send_response();
       break;
-      
+
     }
   }
 }
@@ -966,6 +1020,8 @@ process_requests()
 void
 spawn_child() {
 
+#if defined(HAVE_FORK)
+
   if (debug) {
     fprintf(where,
 	    "%s: enter\n",
@@ -973,7 +1029,6 @@ spawn_child() {
     fflush(where);
   }
 
-#if defined(HAVE_FORK)
 
   /* flush the usual suspects */
   fflush(stdin);
@@ -1032,14 +1087,22 @@ spawn_child() {
   PROCESS_INFORMATION pi;
   STARTUPINFO si;
   int i;
-	    
+
+  if (debug) {
+    fprintf(where,
+	    "%s: enter\n",
+	    __FUNCTION__);
+    fflush(where);
+  }
+
+
   /* create the cmdline array based on strlen(program) + 80 chars */
   cmdline_length = strlen(program) + 80;
   cmdline = malloc(cmdline_length + 1);  // +1 for trailing null
-	    
+
   memset(&si, 0 , sizeof(STARTUPINFO));
   si.cb = sizeof(STARTUPINFO);
-	    
+
   /* Pass the server_sock as stdin for the new process.  Hopefully
      this will continue to be created with the OBJ_INHERIT
      attribute. */
@@ -1047,7 +1110,7 @@ spawn_child() {
   si.hStdOutput = GetStdHandle(STD_OUTPUT_HANDLE);
   si.hStdError = GetStdHandle(STD_ERROR_HANDLE);
   si.dwFlags = STARTF_USESTDHANDLES;
-	    
+
   /* Build cmdline for child process */
   strcpy(cmdline, program);
   cmd_index = strlen(cmdline);
@@ -1077,7 +1140,7 @@ spawn_child() {
 			cmdline_length - cmd_index,
 			" -i %x",
 			(int)(UINT_PTR)where);
-	    
+
   b = CreateProcess(NULL,    /* Application Name */
 		    cmdline,
 		    NULL,    /* Process security attributes */
@@ -1095,13 +1158,13 @@ spawn_child() {
       free(cmdline); /* even though we exit :) */
       exit(1);
     }
-	    
+
   /* We don't need the thread or process handles any more;
      let them go away on their own timeframe. */
-	    
+
   CloseHandle(pi.hThread);
   CloseHandle(pi.hProcess);
-	    
+
   /* the caller/parent will close server_sock */
 
   free(cmdline);
@@ -1177,7 +1240,8 @@ void
 accept_connections() {
 
   fd_set read_fds, write_fds, except_fds;
-  int high_fd, num_ready, candidate;
+  SOCKET high_fd, candidate;
+  int num_ready;
 
   if (debug) {
     fprintf(where,
@@ -1192,12 +1256,26 @@ accept_connections() {
     FD_ZERO(&except_fds);
     high_fd = set_fdset(listen_list,&read_fds);
 
+#if !defined(WIN32)
     num_ready = select(high_fd + 1,
+#else
+    num_ready = select(1,
+#endif
 		       &read_fds,
 		       &write_fds,
 		       &except_fds,
 		       NULL);
 
+    if (num_ready < 0) {
+      fprintf(where,
+	      "%s: select failure: %s (errno %d)\n",
+	      __FUNCTION__,
+	      strerror(errno),
+	      errno);
+      fflush(where);
+      exit(1);
+    }
+
     /* try to keep things simple */
     candidate = 0;
     while ((num_ready) && (candidate <= high_fd)) {
@@ -1210,23 +1288,13 @@ accept_connections() {
 	candidate++;
       }
     }
-
-    if (num_ready < 0) {
-      fprintf(where,
-	      "%s: select failure: %s (errno %d)\n",
-	      __FUNCTION__,
-	      strerror(errno),
-	      errno);
-      fflush(where);
-      exit(1);
-    }
   }
 }
 
 #ifndef WIN32
-#define SERVER_ARGS "DdfhL:n:Np:v:V46"
+#define SERVER_ARGS "DdfhL:n:Np:v:VZ:46"
 #else
-#define SERVER_ARGS "DdfhL:n:Np:v:V46I:i:"
+#define SERVER_ARGS "DdfhL:n:Np:v:VZ:46I:i:"
 #endif
 void
 scan_netserver_args(int argc, char *argv[]) {
@@ -1297,6 +1365,21 @@ scan_netserver_args(int argc, char *argv[]) {
       strncpy(listen_port,optarg,sizeof(listen_port));
       not_inetd = 1;
       break;
+    case 'Z':
+      /* only copy as much of the passphrase as could fit in the
+	 test-specific portion of a control message. Windows does not
+	 seem to have a strndup() so just malloc and strncpy it.  we
+	 weren't checking the strndup() return so won't bother with
+	 checking malloc(). we will though make certain we only
+	 allocated it once in the event that someone puts -Z on the
+	 command line more than once */
+      if (passphrase == NULL) 
+	passphrase = malloc(sizeof(netperf_request.content.test_specific_data));
+      strncpy(passphrase,
+	      optarg,
+	      sizeof(netperf_request.content.test_specific_data));
+      passphrase[sizeof(netperf_request.content.test_specific_data) - 1] = '\0';
+      break;
     case '4':
       local_address_family = AF_INET;
       break;
@@ -1315,7 +1398,7 @@ scan_netserver_args(int argc, char *argv[]) {
       printf("Netperf version %s\n",NETPERF_VERSION);
       exit(0);
       break;
-      
+
     }
   }
 }
@@ -1367,7 +1450,7 @@ daemonize() {
 
       /* ok, we can start accepting control connections now */
       accept_connections();
-      
+
   default:
     /* we are the parent, nothing to do but exit? */
     exit(0);
@@ -1452,7 +1535,7 @@ main(int argc, char *argv[]) {
 
 #ifdef WIN32
   WSADATA	wsa_data ;
-  
+
   /* Initialize the winsock lib do we still want version 2.2? */
   if ( WSAStartup(MAKEWORD(2,2), &wsa_data) == SOCKET_ERROR ){
     printf("WSAStartup() failed : %lu\n", GetLastError()) ;
@@ -1467,15 +1550,15 @@ main(int argc, char *argv[]) {
     return -1 ;
   }
   strcpy(program, argv[0]);
-  
+
   init_netserver_globals();
 
   netlib_init();
-  
+
   strncpy(local_host_name,"",sizeof(local_host_name));
   local_address_family = AF_UNSPEC;
   strncpy(listen_port,TEST_PORT,sizeof(listen_port));
-	  
+
   scan_netserver_args(argc, argv);
 
   check_if_inetd();
@@ -1513,5 +1596,5 @@ main(int argc, char *argv[]) {
 #endif
 
   return 0;
-	  
+
 }
