@@ -1,5 +1,5 @@
 /*
-        Copyright (C) 1993-2010 Hewlett-Packard Company
+        Copyright (C) 1993-2012 Hewlett-Packard Company
 */
 
 /* libraried performance include file 				*/
@@ -77,6 +77,8 @@
 extern char		*program; /* program invocation name		*/
 extern char             *command_line;  /* how we were invoked          */
 
+extern char  *passphrase;
+
 /* stuff to say where this test is going                                */
 extern char	host_name[HOSTNAMESIZE];/* remote host name or ip addr  */
 extern char     local_host_name[HOSTNAMESIZE];
@@ -106,7 +108,7 @@ extern int
 
 /* the end-test conditions for the tests - either transactions, bytes,  */
 /* or time. different vars used for clarity - space is cheap ;-)        */
-extern int	
+extern int
   test_time,		/* test ends by time			*/
   test_len_ticks,
   test_bytes,		/* test ends on byte count		*/
@@ -138,13 +140,6 @@ extern	int	     interval_burst;
 extern  int          remote_interval_usecs;
 extern  int          remote_interval_burst;
 
-#if defined(WANT_INTERVALS) || defined(WANT_DEMO)
-
-extern int    demo_mode;
-extern double demo_interval;
-extern double demo_units;
-extern double units_this_tick;
-#endif 
 
 #ifdef DIRTY
 extern int	rem_dirty_count;
@@ -164,14 +159,21 @@ extern double interval_pct;
 
 extern int cpu_binding_requested;
 
-/* stuff to controll the bufferspace "width" */
+/* stuff to control the bufferspace "width" */
 extern int	send_width;
 extern int      recv_width;
 
+/* control the socket priority */
+extern int local_socket_prio;
+extern int remote_socket_prio;
+
+extern int local_socket_tos;
+extern int remote_socket_tos;
+
 /* address family */
 extern int	af;
 
-/* different options for other things					*/
+/* different options for other things */
 extern int
   local_cpu_usage,
   remote_cpu_usage;
@@ -183,11 +185,12 @@ extern float
 extern int
   shell_num_cpus;
 
-extern	char	
+extern	char
   test_name[BUFSIZ];
 
 extern char
-  fill_file[BUFSIZ];
+  local_fill_file[BUFSIZ],
+  remote_fill_file[32];
 
 extern char *
   result_brand;
@@ -207,3 +210,7 @@ extern int
 #endif /* WANT_DLPI */
 
 #endif
+
+extern int parse_ipqos(const char *cp);
+extern const char * iptos2str(int iptos);
+
